{% extends "_base.html" %} {% block content %}
<h1>Authorize Client</h1>
<p>Client ID: <code>{{ client_id }}</code></p>
<form method="post">
  {% if sub_missing %}
  <input
    type="text"
    name="sub"
    placeholder="sub"
    required
    autofocus
    aria-invalid="true"
    aria-describedby="sub-missing"
  />
  <small id="sub-missing">The field is missing</small>
  {% else %}
  <input type="text" name="sub" placeholder="sub" required autofocus />
  {% endif %}
  <button type="submit">Authorize</button>
</form>
<form method="post">
  <button type="submit" name="action" value="deny">Deny</button>
</form>
<p>You will be redirected to {{ redirect_uri }}</p>
{% endblock %}
